var noScroll_bodyEvent = function(e) {
    if (e.target.id == 'el') return;
    e.preventDefault();
    e.stopPropagation();
};
$(document).ready(function() {
    $("graphic").each(function () {
        $(this).width($(this).attr("width")).height($(this).attr("height"));
    });
});
function _BASE_(_title, _color)
{
    this.isHover = false;
    this._type = null;
    this.title = _title || 'untitled';
    this.style =
    {
        color: _color || '#333'
    },
    this.hover_style =
    {
        color: _color || '#333'
    },
    this.label = 
    {
        obj: null,
        text: function(value) 
        {
            this.obj.children(".value").text(value);
            return this;
        },
        hide: function() {
            this.obj.hide();
            return this;
        },
        show: function() {
            this.obj.show();
            return this;
        }
    }
}
//  _BASE_Shape
//  Base class for graphic shapes
//      x - shape X-axis coordinate
//      y - shape Y-axis coordinate
//      _title - title of shape
//      _color - color of shape on canvas (any format for html)
function _BASE_Shape(x, y, _title, _color) 
{
    _BASE_.call(this, _title, _color);
    this._type = "shape";
    this.__type = null;
    this.coordinats = {
        x: x || 0,
        y: y || 0
    }
    this.style.border = {
        width: 0,
        color: '#333'
    }
    this.hover_style.border = {
        width: 0,
        color: '#aaa'
    }
}
//  rectangleShape
//  Class for graphic rectangle
//      x - rectangle left X-axis coordinate
//      y - rectangle top Y-axis coordinate
//      widht - rectangle width
//      height - rectangle height
//      _title - title of shape
//      _color - color of shape on canvas (any format for html)
function rectangleShape(x, y, width, height, _title, _color) 
{
    _BASE_Shape.call(this, x , y, _title || "Rect", _color);
    this.__type = "rectangle";
    this.width = width || 0;
    this.height = height || 0;
}
//  pointShape
//  Class for graphic rectangle
//      x - point X-axis coordinate
//      y - point Y-axis coordinate
//      _title - title of shape
//      _color - color of shape on canvas (any format for html)
function pointShape(x, y, _title, _color) 
{
    _BASE_Shape.call(this, x , y, _title || "Point", _color);
    this.__type = "point";
}
//  _BASE_GraphicFunc
//  Base class for graphic functions
//      _title - title of function
//      _color - color of funciton on graphic canvas (any format for html)
function _BASE_GraphicFunc(_title, _color) 
{
    _BASE_.call(this, _title, _color);
    this._type = "function";
    this.__type = null;
    this.func =  null; 
    this.values_by_ctx_x = [];
    this.values = [];
    this.is_draw_axis = true;
    this.bounds = {
        xl: null,
        xr: null
    }
}
//  simpleGraphicFunc
//  Graphic function based on JS functions
//      _func - JS function with prototype function(x) { ... return x; } to draw
//      _title - title of function
//      _color - color of funciton on graphic canvas (any format for html)
function simpleGraphicFunc(_func, _title, _color)
{
    _BASE_GraphicFunc.call(this, _title || "Sf", _color, "function");
    this.__type = "simple";
    this.func = _func || function (x) { return x; } 
}
//  polyLineGraphicFunc
//  Poly-line function based on array of points
//      _points - array of objects with prorotype { x: value, y: value }
//      _title - title of function
//      _color - color of funciton on graphic canvas (any format for html)
function polyLineGraphicFunc(_points, _title, _color) 
{
    _BASE_GraphicFunc.call(this, _title || "Plf", _color);
    this.__type = "polyline";
    this.points = _points || [];
    this.func = function (x) 
    { 
        if (this.points.length < 2) return;
        if (x < this.points[0].x || x > this.points[this.points.length-1].x) return 0;
        var x1, x2, y1, y2;
        for (var i = 0; i < this.points.length - 1; i++)
            if (x < this.points[i+1].x)
            {
                x1 = this.points[i].x;
                x2 = this.points[i+1].x;
                y1 = this.points[i].y;
                y2 = this.points[i+1].y;
                return -(( x1 * y2 - x2 * y1) + (y1 - y2) * x) / (x2 - x1);
            }
    } 
}
//  pointGraphicFunc
//  Point function based on point coordinates
//      _x - point coordinate on OX-axis
//      _y - point coordinate on OY-axis
//      _title - title of function
//      _color - color of funciton on graphic canvas (any format for html)
function pointGraphicFunc( _x, _y, _title, _color)
{
    _BASE_GraphicFunc.call(this, _title, _color);
    this.__type = "point";
    this.X = _x || 0;
    this.Y = _y || 0;
}
//  Graphic
//  Graphic class for create and interact with canvas
//      graph_selector - jQuery selector of element for graphic
//      xl - minimum of OX-axis
//      xr - maximum of OX-axis
//      yb - minimum of OY-axis
//      yt - maximum of OY-axis
//      title - title of graphic to show
function Graphic(graph_selector, xl, xr, yb, yt, title, interact)
{
    if (xl >= xr || yb >= yt) return null;
    this.graph = $(graph_selector);
    if(this.graph[0] === undefined) return null;
    this.graph = $(graph_selector).eq(0);
    this.title = {
        text: null,
        obj: null,
        set: function(text) {
            if (!text || text == " ") this.obj.hide();
            else this.obj.html(text);
            this.text = text;
        }
    };
    this.Round = {
        percision: 10000,
        Number: function(n , percision) {
            percision = percision || this.percision;
            return Math.round(n * percision) / percision;
        }
    };
    this.empty = function() {
        this.graph.attr("class","");
        this.graph.empty();
    }
    this.graph.empty();
    this.graph.append(this.title.obj = $('<header/>'));
    this.title.set(title);
    this.graph.append('<div class="functions_vals"></div>');
    this.graph.append('<div class="additional_text"></div>');
    this.graph.append($('<canvas class="canvas_graph"></canvas>').attr("width",this.graph.attr("width")).attr("height",this.graph.attr("height")));
    this.graph.append($('<canvas class="canvas_axis"></canvas>').attr("width",this.graph.attr("width")).attr("height",this.graph.attr("height")));
    this.graph.width(this.graph.attr("width")).height(this.graph.attr("height"));
    var self = this;
    this.style = {
        axis: { 
            color: '#777',
            marks_color: '#777',
            font: '11px Helvetica',
            show_marks: true,
            hide_marks: function() {
                this.show_marks = false;
                self.drawAxis();
            }
        },
        point_axis: {
            color: '#ccc',
            text_color: '#888',
            font: '11px Helvetica'
        }
    };
    this.info = {
        cursorPosition: {
            x: 0,
            y: 0,
            old_x: 0,
            old_y: 0
        },
        isMouseDown: false
    }
    this.graphic_objects = [];
    this.bounds = {
        xl: xl,
        xr: xr,
        yb: yb,
        yt: yt,
        set: function(xl, xr, yb, yt) {
            if (xl >= xr || yb >= yt) return null;
            self.init(xl, xr, yb, yt);
            self.drawAxis();
            self.ReDrawObjects();
        }
    };
    this.axis_canvas = this.graph.find(".canvas_axis")[0];
    this.axis_ctx = this.axis_canvas.getContext("2d");
    this.graph_canvas = this.graph.find(".canvas_graph")[0];
    this.graph_ctx = this.graph_canvas.getContext("2d");
    this.init(xl, xr, yb, yt);
    this.function_values = {
        obj: this.graph.find(".functions_vals"),
        show: function () 
        {
            this.obj.show();
        },
        hide: function () 
        {
            this.obj.hide(200);
        }
    };
    this.additional_text = 
    {
        obj: this.graph.find(".additional_text"),
        visible: false,
        delay: 1500,
        timer: null,
        was_point: false,
        text: "",
        clear: function(at) {
            at.visible = false;
            at.was_point = false;
            at.obj.removeClass("show");
            at.text = "";
        },
        blink: function() {
            if (this.timer) clearTimeout(this.timer);
            this.obj.addClass("show");
            var self = this;
            this.timer = setTimeout(function() { self.clear(self); }, this.delay);
        },
        addChar: function(c) {
            if (c == '.')
            {
                if (this.was_point)
                {
                    this.text = this.text.substr(this.text.indexOf('.')+1,this.text.length);
                    this.was_point = false;
                }
                if (this.text == "")
                    this.text = "0";
                this.was_point = true;
            }
            this.text += c;
            this.obj.text(this.text);
            self.showPointInfoC(self.convertCoordinatsNtoC(parseFloat(this.text),null).c_x);
            this.blink();
        }
    };
    Graphic.graph_mousewheel = function(event, delta) {
        var mult;
        var mult = (delta > 0) ? 0.95 : 1.05;
        var new_width = self.size.width * mult;
        var new_height = self.size.height * mult;
        var CurMCG = self.convertCoordinatsCtoN(self.info.cursorPosition.x, self.info.cursorPosition.y);
        var new_xl = CurMCG.x - self.info.cursorPosition.x / self.graph.width() * new_width;
        var new_xr = new_xl + new_width;
        var new_yb = CurMCG.y - (self.graph.height()-self.info.cursorPosition.y) / self.graph.height() * new_height;
        var new_yt = new_yb + new_height;
        self.bounds.set(new_xl, new_xr, new_yb, new_yt);
        self.showPointInfoC(self.info.cursorPosition.x,self.info.cursorPosition.y);
    };
    Graphic.graph_mousemove = function (e) {
        var parentOffset = $(this).offset();
        self.info.cursorPosition.x = e.pageX - parentOffset.left;
        self.info.cursorPosition.y = e.pageY - parentOffset.top;
        if (self.info.isMouseDown)
        {
            var delta_x = (self.info.cursorPosition.old_x - self.info.cursorPosition.x) / self.step.width;
            var delta_y = (self.info.cursorPosition.y - self.info.cursorPosition.old_y) / self.step.height;
            self.bounds.set(self.bounds.xl+delta_x, self.bounds.xr+delta_x, self.bounds.yb+delta_y, self.bounds.yt+delta_y);
        }
        else
            self.showPointInfoC(self.info.cursorPosition.x,self.info.cursorPosition.y);
        self.info.cursorPosition.old_x = self.info.cursorPosition.x;
        self.info.cursorPosition.old_y = self.info.cursorPosition.y;
    };
    Graphic.graph_mouseleave = function(e) {
        $('body').off('mousewheel', noScroll_bodyEvent);
        self.DrawAxisWithPointsAxis();
        self.function_values.hide();
        self.info.isMouseDown = false;
    };
    Graphic.graph_mouseenter = function(e) {
        $('body').on('mousewheel', noScroll_bodyEvent);
        $(this).attr("tabindex",0);
        self.function_values.show();
    };
    Graphic.graph_mousedown = function(e) {
        self.info.isMouseDown = true;
    };
    Graphic.graph_mouseup = function(e) {
        self.info.isMouseDown = false;
    };
    Graphic.graph_keydown = function(e) {
        if (e.keyCode == 8)
            self.additional_text.clear(self.additional_text);
        else if (e.keyCode >= 48 && e.keyCode <= 57)
            self.additional_text.addChar(String.fromCharCode(e.keyCode));
        else if (e.keyCode == 190)
            self.additional_text.addChar('.');
    };
    this.interactive = {
        isInteractive: null,
        on: function() {
            this.isInteractive = true;
            //self.graph.bind("mousewheel", Graphic.graph_mousewheel);
            self.graph.bind("mousemove", Graphic.graph_mousemove);
            self.graph.bind("mouseleave", Graphic.graph_mouseleave);
            self.graph.bind("mouseenter", Graphic.graph_mouseenter);
            //self.graph.bind("mousedown", Graphic.graph_mousedown);
            self.graph.bind("mouseup", Graphic.graph_mouseup);
            self.graph.bind("keydown", Graphic.graph_keydown);
        },
        off: function() {
            this.isInteractive = false;
            self.graph.unbind("mousewheel", Graphic.graph_mousewheel);
            self.graph.unbind("mousemove", Graphic.graph_mousemove);
            self.graph.unbind("mouseleave", Graphic.graph_mouseleave);
            self.graph.unbind("mouseenter", Graphic.graph_mouseenter);
            self.graph.unbind("mousedown", Graphic.graph_mousedown);
            self.graph.unbind("mouseup", Graphic.graph_mouseup);
            self.graph.unbind("keydown", Graphic.graph_keydown);
        }
    };
    if (interact === undefined || interact)
        this.interactive.on();
    else 
        this.interactive.isInteractive = false;
    this.drawAxis();
}
Graphic.prototype.init = function (xl, xr, yb, yt) 
{
    this.bounds.xl = xl;
    this.bounds.xr = xr;
    this.bounds.yb = yb;
    this.bounds.yt = yt;
    this.size = { 
        width: Math.abs(xr-xl),
        height: Math.abs(yt-yb)
    };
    this.step = {
        width: this.axis_canvas.width / this.size.width,
        height: this.axis_canvas.height / this.size.height
    };
    this.axisO = { 
        c_x: ((xl > 0) ? -1 : 1) * Math.abs(xl)/Math.abs(xr-xl) * this.axis_canvas.width,
        c_y: ((yt < 0) ? -1 : 1) * Math.abs(yt)/Math.abs(yt-yb) * this.axis_canvas.height 
    };
};
Graphic.prototype.convertCoordinatsNtoC = function(x, y) 
{
    return { 
        c_x: (x == null) ? 0 : Math.round(this.axisO.c_x + x * this.step.width),
        c_y: (y == null) ? 0 : Math.round(this.axisO.c_y - y * this.step.height)
    }
};
Graphic.prototype.convertCoordinatsCtoN = function(c_x, c_y) 
{
    return { 
        x: (c_x == null) ? 0 : (c_x - this.axisO.c_x) / this.step.width,
        y: (c_y == null) ? 0 : - (c_y - this.axisO.c_y) / this.step.height
    }
};
Graphic.prototype.convertSizeNtoC = function(x, y) 
{
    return { 
        c_x: (x == null) ? 0 : x * this.step.width,
        c_y: (y == null) ? 0 : y * this.step.height
    }
};
Graphic.prototype.convertSizeCtoN = function(c_x, c_y) 
{
    return { 
        x: (c_x == null) ? 0 : c_x / this.step.width,
        y: (c_y == null) ? 0 : c_y / this.step.height
    }
};
Graphic.prototype.drawAxis = function() 
{
    this.axis_ctx.clearRect(0,0,this.graph_canvas.width,this.graph_canvas.height);
    this.axis_ctx.beginPath();
    this.axis_ctx.strokeStyle = this.style.axis.color;
    this.axis_ctx.moveTo(0,this.axisO.c_y);
    this.axis_ctx.lineTo(this.axis_canvas.width, this.axisO.c_y);
    this.axis_ctx.moveTo(this.axisO.c_x,0);
    this.axis_ctx.lineTo(this.axisO.c_x,this.axis_canvas.height);
    this.axis_ctx.stroke();
    this.axis_ctx.closePath();
    if (this.style.axis.show_marks)
    {
        this.axis_ctx.beginPath();
        this.axis_ctx.fillStyle = this.style.axis.marks_color;
        this.axis_ctx.font = this.style.axis.font;
        this.axis_ctx.textAlign = 'center';

        var gr_delta = this.convertSizeCtoN(35,null).x;
        for (var i = gr_delta; i <= this.bounds.xr; i += gr_delta)
        {
            this.axis_ctx.fillRect(x = this.axisO.c_x + i*this.step.width, this.axisO.c_y-3, 1, 6);
            this.axis_ctx.fillText(Math.round(i*100)/100,x, this.axisO.c_y-5);
        }
        for (var i = -gr_delta; i >= this.bounds.xl; i -= gr_delta)
        {
            this.axis_ctx.fillRect(x = this.axisO.c_x + i*this.step.width, this.axisO.c_y-3, 1, 6);
            this.axis_ctx.fillText(Math.round(i*100)/100,x, this.axisO.c_y+15);
        }
        gr_delta = this.convertSizeCtoN(null, 35).y;
        this.axis_ctx.textAlign = 'left';
        for (var i = gr_delta; i <= this.bounds.yt; i += gr_delta)
        {
            this.axis_ctx.fillRect(this.axisO.c_x-3, y = this.axisO.c_y - i*this.step.height, 6, 1);
            this.axis_ctx.fillText(Math.round(i*100)/100,this.axisO.c_x+7, y+5);
        };
        this.axis_ctx.textAlign = 'right';
        for (i = -gr_delta; i >= this.bounds.yb; i -= gr_delta)
        {
            this.axis_ctx.fillRect(this.axisO.c_x-3, y = this.axisO.c_y - i*this.step.height, 6, 1);
            this.axis_ctx.fillText(Math.round(i*100)/100,this.axisO.c_x-10, y+5);
        }

        this.axis_ctx.closePath();
    }
};
Graphic.prototype.AddLabel = function(obj)
{
    if (obj.__type == null) return;
    this.function_values.obj.append('<label>'+
                            '<div class="point" style="background-color: '+ obj.style.color +';"></div>'+
                            '<div class="value">-</div>'+
                            '</label>');
    return this.function_values.obj.find("label").last();
};
Graphic.prototype.showPointInfoC = function(relX, relY, showTarget) 
{
    var show_shapes = relY !== undefined;
    var redraw = false;
    var self = this;
    showTarget = (showTarget === undefined) ? true : showTarget;
    relX = relX || 0;
    relY = relY || 0;
    var point = this.convertCoordinatsCtoN(relX,relY);
    var points = [];
    if (showTarget)
        points.push({x: point.x, y: point.y });
    var cur_c_x;
    var prevHover;
    this.graphic_objects.forEach(function( obj ) {
        if (obj._type == "function")
            if (obj.is_draw_axis && 
            obj.values_by_ctx_x[0].ctx_x <= relX &&
            obj.values_by_ctx_x[obj.values_by_ctx_x.length-1].ctx_x >= relX)
            {
                cur_c_x = Math.round(relX-obj.values_by_ctx_x[0].ctx_x);
                points.push({ 
                    x: self.convertCoordinatsCtoN(relX,null).x,
                    y: obj.values_by_ctx_x[cur_c_x].normal_value
                });
                obj.label.text(obj.title + ": " + self.Round.Number(points[points.length-1].y));
            }
            else
                obj.label.text(obj.title + ": out of field bounds");
        else if (obj._type == "shape" && show_shapes)
        {
            prevHover = obj.isHover;
            if (obj.__type == "rectangle")
                if (point.x >= obj.coordinats.x &&
                    point.x <= obj.coordinats.x + obj.width && 
                    point.y >= obj.coordinats.y &&
                    point.y <= obj.coordinats.y + obj.height)
                {
                    obj.label.show().text(obj.title + " (" + 
                                          self.Round.Number(obj.coordinats.x, 100) + "," + 
                                          self.Round.Number(obj.coordinats.y, 100) + ") " + 
                                          self.Round.Number(obj.width, 100) + " x " + 
                                          self.Round.Number(obj.height, 100));
                    obj.isHover = true;
                }
                else 
                {
                    obj.label.hide();
                    obj.isHover = false;
                }
            else if (obj.__type == "point")
            {
                var delta_x =  self.convertSizeCtoN(4,null).x;
                var delta_y =  self.convertSizeCtoN(null,4).y;
                if (point.x >= obj.coordinats.x - delta_x && 
                    point.x <= obj.coordinats.x + delta_x &&
                    point.y >= obj.coordinats.y - delta_y && 
                    point.y <= obj.coordinats.y + delta_y)
                    obj.label.show().text(obj.title + " (" + 
                                          self.Round.Number(obj.coordinats.x, 100) + "," + 
                                          self.Round.Number(obj.coordinats.y, 100) + ") ");
                else 
                    obj.label.hide();
            }
            redraw = redraw || (obj.isHover != prevHover);
        }    
    });
    if (redraw) this.ReDrawObjects();
    this.DrawAxisWithPointsAxis(points);
};
Graphic.prototype.showPointInfoN = function(X, Y, showTarget)
{
    var point = this.convertCoordinatsNtoC(X, Y);
    this.showPointInfoC(point.c_x, point.c_y, showTarget);
};
Graphic.prototype.DrawAxisWithPointsAxis = function(points) 
{
    this.drawAxis();
	console.log(points);
    if (points && points.length > 1)
    {
        for (var i = 1; i<points.length; i++)
        {
            var point = this.convertCoordinatsNtoC(points[i].x,points[i].y);
            this.axis_ctx.beginPath();
            this.axis_ctx.strokeStyle = this.style.point_axis.color;
            this.axis_ctx.moveTo(0,point.c_y);
            this.axis_ctx.lineTo(this.axis_canvas.width,point.c_y);
            this.axis_ctx.moveTo(point.c_x,0);
            this.axis_ctx.lineTo(point.c_x,this.axis_canvas.height);
            this.axis_ctx.stroke();
            this.axis_ctx.closePath();
            this.axis_ctx.beginPath();
            this.axis_ctx.font = this.style.point_axis.font;
            this.axis_ctx.fillStyle = this.style.point_axis.text_color;
            this.axis_ctx.textAlign = 'left';
            this.axis_ctx.fillText('X : ' + Math.round(points[i].x*10000)/10000, point.c_x+5, point.c_y-17);
            this.axis_ctx.fillText('Y : ' + Math.round(points[i].y*10000)/10000, point.c_x+5, point.c_y-5);
            this.axis_ctx.stroke();
            this.axis_ctx.closePath();
        }
    }
};
Graphic.prototype.AddObject = function(obj)
{
    obj.label.obj = this.AddLabel(obj);
    this.graphic_objects.push(jQuery.extend(true, {}, obj));
    var result;
    this.ReDrawObjects();
};
Graphic.prototype.ReDrawObjects = function() {
    this.clearObjectsCanvas();
    var self = this;
    this.graphic_objects.forEach(function(obj) {
        self.DrawObject(obj);
    });
};
Graphic.prototype.DrawObject = function(obj) {
    if (obj._type == "shape")
        this.DrawShape(obj);
    else if (obj._type == "function")
        this.DrawFunction(obj);
};
Graphic.prototype.DrawShape = function(shape)
{
    if (shape.__type == "rectangle")
        this.DrawRectgangle(shape);
    else if (shape.__type == "point")
        this.DrawPoint(shape);
};
Graphic.prototype.DrawFunction = function(f)
{
    if (f.__type == "simple")
        this.drawSimpleFunction(f);
    else if (f.__type == "polyline")
        this.drawPolyLineFunction(f);
};
Graphic.prototype.DrawRectgangle = function(rec) 
{
    if (rec === undefined || rec.width == 0 || rec.height == 0) return null;
    var coords = this.convertCoordinatsNtoC(rec.coordinats.x, rec.coordinats.y);
    var size = this.convertSizeNtoC(rec.width, rec.height);
    var b_w = rec.style.border.width;
    this.graph_ctx.beginPath();
    this.graph_ctx.rect(coords.c_x - b_w, coords.c_y - size.c_y - b_w, size.c_x - b_w, size.c_y - b_w);
    if (rec.style.border.width > 0)
    {
        this.graph_ctx.strokeStyle = rec.style.border.color;
        this.graph_ctx.strokeWidth = rec.style.border.width;
        this.graph_ctx.stroke();
    }
    this.graph_ctx.fillStyle = (rec.isHover) ? rec.hover_style.color: rec.style.color;
    this.graph_ctx.fill();
    this.graph_ctx.closePath();
};
Graphic.prototype.DrawPoint = function(point) 
{
    var ctx_point = this.convertCoordinatsNtoC(point.coordinats.x,point.coordinats.y);
    this.graph_ctx.beginPath();
    this.graph_ctx.arc(ctx_point.c_x, ctx_point.c_y, 3, 0, 2 * Math.PI , false);
    if (point.style.border.width > 0)
    {
        this.graph_ctx.strokeStyle = point.style.border.color;
        this.graph_ctx.lineWidth = point.style.border.width;
        this.graph_ctx.stroke();
    }
    this.graph_ctx.fillStyle = (point.isHover) ? point.hover_style.color: point.style.color;
    this.graph_ctx.fill();
    this.graph_ctx.closePath();
};
Graphic.prototype.drawSimpleFunction = function(g_f)
{
    var xl = g_f.bounds.xl;
    var xr = g_f.bounds.xr;
    xl = (xl == 0) ? 0 : xl || this.bounds.xl;
    xr = (xr == 0) ? 0 : xr || this.bounds.xr;
    this.graph_ctx.beginPath();
    this.graph_ctx.strokeStyle = g_f.style.color;
        this.graph_ctx.lineWidth = 1.5;
    var c_xl = this.convertCoordinatsNtoC(xl,null).c_x;
    var c_xr = this.convertCoordinatsNtoC(xr,null).c_x;
    g_f.values_by_ctx_x.length = 0;
    g_f.values_by_ctx_x[0] = { 
        ctx_x: c_xl, 
        ctx_y: this.convertCoordinatsNtoC(null,g_f.func(this.convertCoordinatsCtoN(c_xl,null).x)).c_y 
    }; 
    this.graph_ctx.moveTo(g_f.values_by_ctx_x[0].c_x, g_f.values_by_ctx_x[0].c_y);
    for ( var i=c_xl+1, value = 0; i <= c_xr; i++)
    {
        value = g_f.func(this.convertCoordinatsCtoN(i,null).x);
        g_f.values_by_ctx_x.push({ 
            ctx_x: i, 
            normal_value: value,
            ctx_y: this.convertCoordinatsNtoC(null,value).c_y,
        }); 
        this.graph_ctx.lineTo(g_f.values_by_ctx_x[g_f.values_by_ctx_x.length - 1].ctx_x, g_f.values_by_ctx_x[g_f.values_by_ctx_x.length - 1].ctx_y);
    }
    this.graph_ctx.stroke();
    this.graph_ctx.closePath();
};
Graphic.prototype.drawPolyLineFunction = function(g_f)
{
    if (g_f.points == null || g_f.points.length < 2) return null;
    g_f.values_by_ctx_x.length = [];
    this.graph_ctx.beginPath();
    this.graph_ctx.strokeStyle = g_f.style.color;
    this.graph_ctx.lineWidth = 1.0;
    var point_l = this.convertCoordinatsNtoC(g_f.points[0].x,null);
    var point_r = this.convertCoordinatsNtoC(g_f.points[g_f.points.length-1].x,null);
    for (var x = point_l.c_x+1; x < point_r.c_x; x++)
    {
        value = g_f.func(this.convertCoordinatsCtoN(x,null).x);
        g_f.values_by_ctx_x.push({ 
            ctx_x: x,
            normal_value: value,
            ctx_y: this.convertCoordinatsNtoC(null,g_f.func(this.convertCoordinatsCtoN(x,null).x)).c_y
        });
        this.graph_ctx.lineTo(g_f.values_by_ctx_x[g_f.values_by_ctx_x.length - 1].ctx_x,g_f.values_by_ctx_x[g_f.values_by_ctx_x.length - 1].ctx_y);
    }
    this.graph_ctx.stroke();
    this.graph_ctx.closePath();
};
Graphic.prototype.clearObjects = function()
{
    this.graphic_objects.forEach( function( f ) {
        if ( f.label.obj != null )
            f.label.obj.remove();
    });
    this.graphic_objects = [];
    this.clearObjectsCanvas();
};
Graphic.prototype.clearObjectsCanvas = function()
{
    this.graph_ctx.clearRect(0,0,this.graph_canvas.width,this.graph_canvas.height);
};