//  differencingSchemeC
//  Returns the value of Xi basis on approximate difference scheme function at all points of the partition as an object: { x: value, y: value }
//      xL - left boundary for calculations 
//      xR - right boundary for calculations 
//      targetFunction - approximated function
//      t0 - starting time
//      t1 - final time
//      velocity - particles velocity
//      partition - value of the partition
function differencingSchemeC (xL, xR, targetFunction, t0 , t1, velocity, partition, tau)
{
    var h = Math.abs(xR - xL) / partition;
    var max = targetFunction(xL);
    var points = [];
    var U_i = [];
    var U_next = [];
    var resultValues = [];
    for (var i = 0; i <= partition; i++) 
    {
        points[i] = xL + i * h;
        U_i[i] = targetFunction(points[i]);
        if (targetFunction(points[i]) > max)
            max = targetFunction(points[i]);
    }
    tau = tau || 0.33 * h / max;
    var count = (t1 - t0) / tau;
    for (i = 0; i < count; i++)
    {
        U_next[0] = 0;
        resultValues[0] = { x: points[0], y: 0 };
        for(var j = 1; j <= partition; j++) 
        {
            U_next[j] = velocity * tau * (U_i[j-1] - U_i[j]) / h + U_i[j];
            resultValues[j] = { x: points[j], y: U_next[j] };
        }
        for (var j=0; j <= partition; j++)
            U_i[j] = U_next[j];
    }
    return resultValues;
}
function differencingSchemeQL(xL, xR, targetFunction, t0 , t1, partition, tau)
{
    var h = Math.abs(xR - xL) / partition;
    var max = targetFunction(xL);
    var points = [];
    var U_i = [];
    var U_next = [];
    var resultValues = [];
    for (var i = 0; i <= partition; i++) 
    {
        points[i] = xL + i * h;
        U_i[i] = targetFunction(points[i]);
        if (targetFunction(points[i]) > max)
            max = targetFunction(points[i]);
    }
    tau = h / max/3.0;//tau || 0.33 * h / max;
    var count = (t1 - t0) / tau;
	/*
	for(var i=0;i<=partition;i++)
			console.log(U_i[i]);*/
	//console.log(tau);
    for (i = 0; i < count; i++)
    {
		//console.log(i+' ');
        U_next[0] = 0;
		U_next[partition]=0;
        resultValues[0] = { x: points[0], y: 0 }; 
		resultValues[partition] = { x: points[partition], y: 0 };
        for(var j = 1; j < partition; j++) 
        {
            U_next[j] = (U_i[i-1]*U_i[i-1]-U_i[i]*U_i[i])/(2*h)+U_i[i]//- 0.5*tau*(U_i[j+1] - 2*U_i[j] + U_i[j-1])/h/h + U_i[j];
			if (U_next[j]>3.0) {
				return resultValues;
			}
            resultValues[j] = { x: points[j], y: U_next[j] };
        }
        for (var j=0; j <= partition; j++)
            U_i[j] = U_next[j];
    }
	//console.log(resultValues);
	/*	U_next[0] = 0;
		U_next[partition]=0;
        resultValues[0] = { x: points[0], y: 0 }; 
		resultValues[partition] = { x: points[partition], y: 0 };
        for(var j = 1; j < partition; j++) 
        {
            U_next[j] = - 0.5*tau*(U_i[j+1] - 2*U_i[j] + U_i[j-1])/h/h + U_i[j];
			if (U_next[j]>3.0) {
				console.log("ERROR!");
				console.log(points);
				return resultValues;
			}
            resultValues[j] = { x: points[j], y: U_next[j] };
        }
        for (var j=0; j <= partition; j++)
            U_i[j] = U_next[j];*/
		//console.log(U_i[parseInt(partition/2)]+' '+tau);
    return resultValues;
}
//  particleMethodC
//  Returns the value of Xi basis on approximate with particle method function at all points of the partition as an array of objects: { x: value, y: value }
//      xL - left boundary for calculations 
//      xR - right boundary for calculations 
//      targetFunction - approximated function
//      t0 - starting time
//      t1 - final time
//      velocity - particles velocity
//      partition - value of the partition
function particleMethodC (xL, xR, targetFunction, t0, t1, velocity, partition, tau) 
{
    var h = Math.abs(xR - xL) / partition;
    var max = targetFunction(xL);
    var points = [];
    for (var i = 0; i < partition; i++) 
    {
        points[i] = { x: xL + h * (i + 0.5), y: targetFunction(xL + h * (i + 0.5)) };
        if (points[i].y > max)
            max = points[i].y;
    }
    tau = tau || 0.33 * h / max;
    var count = (t1 - t0) / tau;
    for (var i = 0; i < count; i++)
        for (var j = 0; j < partition; j++)
            points[j].x += velocity * tau;
    return points;
}
//  particleMethodQL
//  class for calculating points after using quasilinear particle method as an array of objects: { x: value, y: value }
//      xL - left boundary for calculations 
//      xR - right boundary for calculations 
//      targetFunction - approximated function
//      t0 - starting time
//      t1 - final time
//      velocity - particles velocity
//      partition - value of the partition
function particleMethodQL(xL, xR, targetFunction, t0, t1, partition)
{
    this.parameters = {
        xL: xL || null,
        xR: xR || null,
        targetFunction: targetFunction || null,
        t0: t0 || null,
        t1: t1 || null,
        partition: partition || 1000,
        h0: null
    };
    this.current_step = {
        points: [],
        number: null,
        t: null,
        tau: null,
        templ: null,
        tempr: null,
        mi: null,
        hi: null,
        minh: null,
        maxy: null,
		error_occured : false//HERE IS FLAG YOU NEED
    };
}
particleMethodQL.prototype.init = function() 
{
    this.current_step.number = 0;
    this.current_step.t = 0;
    this.parameters.h0 = (this.parameters.xR - this.parameters.xL) / this.parameters.partition;
    this.current_step.maxy = this.parameters.targetFunction(this.parameters.xL + 0.5 * this.parameters.h0);
    this.current_step.points[-1] = {
        x: this.parameters.xL-0.5*this.parameters.h0,
        y: 0,
        h: this.parameters.h0
    };
    for (var i = 0; i < this.parameters.partition; i++)
    {
        this.current_step.points[i] = {
            x: this.parameters.xL + this.parameters.h0 * (i + 0.5),
            y: this.parameters.targetFunction(this.parameters.xL + this.parameters.h0 * (i + 0.5)),
            h: this.parameters.h0
        };
        if (this.current_step.points[i].y > this.current_step.maxy) 
            this.current_step.maxy = this.current_step.points[i].y;
    }
    this.current_step.tau = 0.33 * this.parameters.h0 / this.current_step.maxy;
    this.current_step.t = this.parameters.t0;
    this.current_step.templ = 0;
    this.current_step.tempr = 0;
};
particleMethodQL.prototype.doStep = function()
{
	var temp_points = [];
	temp_points[-1] = this.current_step.points[-1];
    if (this.current_step.number == null) {
        this.init();
		return this.current_step.points;
	}
    else if (this.current_step.t >= this.parameters.t1)
        return this.current_step.points;
    for (var i = 0; i < this.parameters.partition; i++) { 
        this.current_step.points[i].x += this.current_step.tau * this.current_step.points[i].y / 2;
		temp_points[i]=this.current_step.points[i];
	}
	//delete this
    this.current_step.maxy = this.current_step.points[0].y;
    this.current_step.minh = this.current_step.points[0].h;
    for (var i = 0; i < this.parameters.partition; i++) 
    {
        this.current_step.hi = this.current_step.points[i].x - this.current_step.points[i-1].x - this.current_step.points[i-1].h / 2.0;
        if (2 * this.current_step.hi != this.current_step.points[i].h) 
        {
            if (i < this.parameters.partition - 1) 
                if (2 * this.current_step.hi > this.current_step.points[i].h &&
                    this.current_step.points[i].x + this.current_step.hi > this.current_step.points[i + 1].x) 
                {
                    this.current_step.templ = this.current_step.points[i].x - this.current_step.hi;
                    this.current_step.tempr = this.current_step.points[i + 1].x - this.current_step.points[i + 1].h / 2.0;
                    // WHAT
                    this.current_step.mi = this.current_step.points[i].h * this.current_step.points.y;
                    temp_points[i].h = this.current_step.tempr - this.current_step.templ;
                    temp_points[i].x = (this.current_step.tempr - this.current_step.templ) / 2.0;
                    temp_points[i].y = this.current_step.mi / temp_points[i].h;
                   
                }
                else
                { 
                    this.current_step.mi = this.current_step.points[i].h * this.current_step.points[i].y;
                    temp_points[i].h = 2 * this.current_step.hi;
                    temp_points[i].y = this.current_step.mi / temp_points[i].h;
                }
            else 
            { 
                this.current_step.mi = this.current_step.points[i].h * this.current_step.points[i].y;
                temp_points[i].h = 2 * this.current_step.hi;
                temp_points[i].y = this.current_step.mi / temp_points[i].h;
            }
        }
		if (temp_points[i].x < temp_points[i-1].x + temp_points[i-1].h/2.0) 
			this.current_step.error_occured = true;
        if (temp_points[i].h < this.current_step.minh) 
            this.current_step.minh = temp_points[i].h;
        if (temp_points[i].y > this.current_step.maxy) 
            this.current_step.maxy = temp_points[i].y;
    }
	for (i=0;i<this.parameters.partition;i++)
		this.current_step.points[i]=temp_points[i];
    this.current_step.t += this.current_step.tau;
    this.current_step.tau = 0.1 * this.current_step.minh / this.current_step.maxy;
    if (this.current_step.tau < 0.0000000000001)
        return undefined;
    this.current_step.number++;
    return this.current_step.points;
}
//  Interference
//  Сlass for calculation interference
//ОЧЕНЬ БЛЯТЬ ВАЖНО!!!
//1) теперь в InterferentionImage хранится массив float
//2) появился новый метод, окрашивающий массив
//3) в этом методе будет вставка значений интенсивности в поле "val", а не "а", как раньше, потому что интенсивность уже учитывается при окрашивании
//и ее не нужно второй раз учитывать, соответственно в графике и в поле, где интенсивность по курсору, бери значение val
function InterAndDiffr()
{
    this.isInitInterferention = false;
    this.isInitDiffraction = false;
    
    this.InterferentionImage = [];
    this.DiffractionImage = [];
    
    this.image_size = null;
    
    this.L = null;
    this.E = null;
    this.la = null;
    this.num = null;
    this.tau = null;
    this.e_sphere = null;
    this.grid_nodes_number = null;
    this.pixel_scale = null;
    this.lin_size = null;
	this.picture_scale = null;
	this.GraphicInterpolated = null;
	
    this.d_L = null;
    this.d_E = null;
	this.d_la = null;
    this.size_m = null;
    this.minIntense = null;
    this.maxIntense = null;
    this.mini = null;
    this.minj = null;
    this.maxi = null;
    this.maxj = null;
    this.scales = null;
    this.fresnel_zones = null;
    
    this.inter_sc;
    this.sc;
    this.max = 0;
}
InterAndDiffr.prototype.MakeColored = function(mas,la) 
{
	var color = this.CountColor(la,0,0,0);
	var ColoredP  = [];
	for (var i=0;i<mas.length;i++)
		ColoredP[i] = [];
	for (var i=0;i<mas.length;i++)
		for (var j=0;j<mas.length;j++) 
		{
			
			ColoredP[i][j] = {
                r: parseInt(color.r * mas[i][j]), 
                g: parseInt(color.g * mas[i][j]), 
                b: parseInt(color.b * mas[i][j]),
                a: 1,
				val:mas[i][j]
			}
			
			/*if ((i==parseInt(471/2))&&(j==parseInt(471/2))) 
			{
				ColoredP[i][j] = {
                r: 255, 
                g: 255, 
                b: 255,
                a: 1,
				val:mas[i][j]
			}*/
			
		}
	return ColoredP;
}
InterAndDiffr.prototype.CalcInter = function(E, la, L, tau, e_sphere, grid_nodes_number, size_x, size_y, lin_size, max_size) 
{
	
	var fresnel_zones1 = (lin_size*lin_size/4.0/3.8/0.2)*10000000;
	console.log(fresnel_zones1+' ' );
	this.lin_size = lin_size;
    this.E = E;
    this.la = la;
    this.L = L;
	//this.num = num
    this.tau = tau;
    this.e_sphere = e_sphere;
    this.grid_nodes_number = grid_nodes_number;
    this.image_size = size_x;
    this.InterferentionImage = [];
    for(var i = 0; i < size_x; i++) this.InterferentionImage[i] = [];
    
    this.pixel_scale = 1.2*max_size/size_x;
	this.picture_scale = 1.2*max_size/size_x;//this.CountScale(this.num, this.la, this.L, size_x);
    this.max = 0.0;
	var sqr_display_radius = lin_size*lin_size/this.pixel_scale/this.pixel_scale/4.0;
    for (var i = 0, res; i < size_x; i++)
        for (var j = 0; j < size_y; j++)
        {
			if ((i - size_x/2.0) * (i - size_x/2.0) + (j - size_x/2.0) * (j - size_x/2.0) >= sqr_display_radius )
                res = 0.0;
			else
				res = this.CountInterMatveev(this.la, this.L, this.E, Math.abs(i - size_x/2.0), Math.abs(j - size_y / 2.0), this.pixel_scale, this.tau, this.e_sphere) / 2.0;
            this.max = (res > this.max) ? res : this.max;
            this.InterferentionImage[i][j] = res;
        }
		for (var i = 0, res; i < size_x; i++)
			for (var j = 0; j < size_y; j++)
				this.InterferentionImage[i][j]/=this.max;
			/*
    for (var i = 0, vis; i < size_x; i++)
        for (var j = 0; j < size_y; j++)
        {
            vis = this.inter_sc[i][j] / this.max;
            this.InterferentionImage[i][j] = {
                r: parseInt(r * vis), 
                g: parseInt(g * vis), 
                b: parseInt(b * vis),
                a: 1,
				val:vis
                
            };
        }*//*
    this.pixel_scale = lin_size/grid_nodes_number;//this.CountScale(this.num, this.la, this.L, this.grid_nodes_number);
	var count_radius = lin_size*lin_size/this.pixel_scale/this.pixel_scale/4.0;
    for (var i = 0, res; i < this.grid_nodes_number; i++)
        for (var j = 0; j < this.grid_nodes_number; j++)
        {
            res = this.CountInterMatveev(this.la, this.L, this.E, Math.abs(i - this.grid_nodes_number / 2.0), Math.abs(j - this.grid_nodes_number / 2.0), this.pixel_scale, this.tau, this.e_sphere) / 2.0;
            if ((i - this.grid_nodes_number / 2) * (i - this.grid_nodes_number / 2) + (j - this.grid_nodes_number / 2) * (j - this.grid_nodes_number / 2) >= count_radius)
                this.inter_sc[i][j] = 0.0;
            else 
                this.inter_sc[i][j] = res / this.max;
        }*/
    this.isInitInterferention = true;
    return this.InterferentionImage;
};
InterAndDiffr.prototype.CalcInterGrid = function() 
{
	this.inter_sc = [];
    for(var i = 0; i <this.grid_nodes_number; i++) this.inter_sc[i] = [];
	console.log(this.grid_nodes_number + '  ' + this.image_size);
	if (this.isInitInterferention){
		this.pixel_scale = this.lin_size/this.grid_nodes_number;//this.CountScale(this.num, this.la, this.L, this.grid_nodes_number);
		var count_radius = this.lin_size*this.lin_size/this.pixel_scale/this.pixel_scale/4.0;
		for (var i = 0, res; i < this.grid_nodes_number; i++)
        for (var j = 0; j < this.grid_nodes_number; j++)
        {
            res = this.CountInterMatveev(this.la, this.L, this.E, Math.abs(i - this.grid_nodes_number / 2.0), Math.abs(j - this.grid_nodes_number / 2.0), this.pixel_scale, this.tau, this.e_sphere) / 2.0;
            if ((i - this.grid_nodes_number / 2) * (i - this.grid_nodes_number / 2) + (j - this.grid_nodes_number / 2) * (j - this.grid_nodes_number / 2) >= count_radius)
                this.inter_sc[i][j] = 0.0;
            else 
                this.inter_sc[i][j] = res / this.max;
        }
	}
	else 
		console.log("Can't count interferention grid");
	console.log(this.fresnel_zones+ ' ');
}
InterAndDiffr.prototype.CalcInterThreePoints = function(E, la, L, tau, e_sphere, grid_nodes_number, size_x, size_y, lin_size) 
{
    this.E = E;
    this.la = la;
    this.L = L;
	//this.num = num
    this.tau = tau;
    this.e_sphere = e_sphere;
    this.grid_nodes_number = grid_nodes_number;
    this.image_size = size_x;
    
    this.inter_sc = [];
    for(var i = 0; i < Math.max(this.grid_nodes_number, size_x); i++) this.inter_sc[i] = [];
    this.InterferentionImage = [];
    for(var i = 0; i < size_x; i++) this.InterferentionImage[i] = [];
    this.pixel_scale = lin_size/size_x;//this.CountScale(this.num, this.la, this.L, size_x);
	var scenter = (size_x/2)*this.pixel_scale;
    var xl = scenter-lin_size/5.0,xr = scenter+lin_size/5.0;
    var color = this.CountColor(la, 0, 0, 0);
    var r = color.r, g = color.g, b = color.b;
    this.max = 0.0;
	var k = 2*Math.PI/this.la;
	var ImPart,RePart,Eleft = 1.0,Eright = 1.0,Eplane = 1.0;
    for (var i = 0, res; i < size_x; i++)
        for (var j = 0; j < size_y; j++)
        {
			ImPart=0.0;
			RePart=0.0;
			ImPart+=Eleft*Math.sin(k*((i*this.pixel_scale-xl)*(i*this.pixel_scale-xl)+(j*this.pixel_scale-scenter)*(j*this.pixel_scale-scenter)+this.L*this.L));
			ImPart+=Eright*Math.sin(k*((i*this.pixel_scale-xr)*(i*this.pixel_scale-xr)+(j*this.pixel_scale-scenter)*(j*this.pixel_scale-scenter)+this.L*this.L));
			ImPart+=Eplane*Math.sin(k*this.L*this.L);
			RePart+=Eleft*Math.cos(k*((i*this.pixel_scale-xl)*(i*this.pixel_scale-xl)+(j*this.pixel_scale-scenter)*(j*this.pixel_scale-scenter)+this.L*this.L));
			RePart+=Eright*Math.cos(k*((i*this.pixel_scale-xr)*(i*this.pixel_scale-xr)+(j*this.pixel_scale-scenter)*(j*this.pixel_scale-scenter)+this.L*this.L));
			RePart+=Eplane*Math.cos(k*this.L*this.L);
			res = ImPart*ImPart + RePart*RePart;
            //res = this.CountInterMatveev(this.la, this.L, this.E, Math.abs(i - size_x / 2 - 1), Math.abs(j - size_y / 2 - 1), this.pixel_scale, this.tau, this.e_sphere) / 2.0;
            this.max = (res > this.max) ? res : this.max;
            this.inter_sc[i][j] = res;
        }
    for (var i = 0, vis; i < size_x; i++)
        for (var j = 0; j < size_y; j++)
        {
            vis = this.inter_sc[i][j] / this.max;
            this.InterferentionImage[i][j] = {
                r: parseInt(r * vis), 
                g: parseInt(g * vis), 
                b: parseInt(b * vis),
                a: 1,
				val:vis
                
            };
        }
   // this.pixel_scale = lin_size/grid_nodes_number;//this.CountScale(this.num, this.la, this.L, this.grid_nodes_number);
  /*  for (var i = 0, res; i < this.grid_nodes_number; i++)
        for (var j = 0; j < this.grid_nodes_number; j++)
        {
			ImPart=0.0;
			RePart=0.0;
			ImPart+=Eleft*Math.sin(k*((i*this.pixel_scale-xl)*(i*this.pixel_scale-xl)+(j*this.pixel_scale)*(j*this.pixel_scale)+this.L*this.L));
			ImPart+=Eright*Math.sin(k*((i*this.pixel_scale-xr)*(i*this.pixel_scale-xr)+(j*this.pixel_scale)*(j*this.pixel_scale)+this.L*this.L));
			ImPart+=Eplane*Math.sin(k*this.L*this.L);
			RePart+=Eleft*Math.cos(k*((i*this.pixel_scale-xl)*(i*this.pixel_scale-xl)+(j*this.pixel_scale)*(j*this.pixel_scale)+this.L*this.L));
			RePart+=Eright*Math.cos(k*((i*this.pixel_scale-xr)*(i*this.pixel_scale-xr)+(j*this.pixel_scale)*(j*this.pixel_scale)+this.L*this.L));
			RePart+=Eplane*Math.cos(k*this.L*this.L);
			res = ImPart*ImPart + RePart*RePart; 
			this.inter_sc[i][j] = res / this.max;
        }*/
    this.isInitInterferention = true;
    return this.InterferentionImage;
};
InterAndDiffr.prototype.CalcDiffr_CircularAperture = function (d_L, d_la, d_radius, image_size)
{
    this.image_size = image_size;
    this.d_L = d_L;
    this.d_E = 10.0;
    this.la = d_la;
    this.fresnel_zones = (d_radius*d_radius/d_la/d_L);//ADD TO >>>
    this.sc = [];
    for(var i = 0; i < this.image_size; i++) this.sc[i] = [];
    this.InterferentionImage = [];
    for(var i = 0; i < this.image_size; i++) this.InterferentionImage[i] = [];
    this.DiffractionImage = [];
    for(var i = 0; i < this.image_size; i++) this.DiffractionImage[i] = [];
    var scr = [];
	if (this.fresnel_zones>50)
		this.grid_nodes_number = 1001;
	else if (this.fresnel_zones>25) 
		this.grid_nodes_number = 501;
	else if (this.fresnel_zones>12) 
		this.grid_nodes_number = 251;
	else 
		this.grid_nodes_number = 201;

    var pixel = 4*d_radius/this.grid_nodes_number;
    this.pixel_scale = pixel*this.grid_nodes_number/this.image_size;
    if (this.fresnel_zones>0.1)
		this.scales = pixel*this.grid_nodes_number/this.image_size;
	else if (this.fresnel_zones>0.01)
		this.scales = 30*pixel*this.grid_nodes_number/this.image_size;
    else 
		this.scales = 120*pixel*this.grid_nodes_number/this.image_size;
	var k = 2.0 * Math.PI / this.la;
    var E0 = this.d_E;
    var color = this.CountColor(this.la, 0, 0, 0);
    var r = color.r, g = color.g, b = color.b;
    var sum, xs, ys, x, y, koe, re, inte, vis;
    this.size_m = this.scales * this.image_size;
    this.minIntense = E0 * E0;
    this.maxIntense = 0.0;
    this.inter_sc = [];
	var pcenter = this.grid_nodes_number/2;//parseInt(this.grid_nodes_number/2)+1;
	var scenter = this.image_size/2;// parseInt(this.image_size/2)+1;
    for (i=0;i<this.grid_nodes_number;i++)
    {
        this.inter_sc[i] = [];
        for (j=0;j<this.grid_nodes_number;j++)
            if ((i-pcenter)*(i-pcenter)+(j-pcenter)*(j-pcenter)<d_radius*d_radius/pixel/pixel)
                  this.inter_sc[i][j]=1.0;
            else this.inter_sc[i][j]=0.0;
    }
    for (var i = 0, vis; i < this.image_size; i++)
        for (var j = 0; j < this.image_size; j++)
        {
            //vis = this.inter_sc[i][j];
            this.InterferentionImage[i][j] = {
                r: 0,//parseInt(r * vis), 
                g: 0,//parseInt(g * vis), 
                b: 0,//parseInt(b * vis),
                a: 0// vis
            };
        }
    var RePart;
	var ImPart , te, te1;
	te1 = pixel*pixel*E0*this.d_L;
	/*for (var i = 0; i < this.image_size; i++)
		for (var j=0; j < this.image_size;j++)
		{
        sum = 0.0;
		RePart = 0.0;
		ImPart = 0.0;
        xs = (i - parseInt(this.image_size / 2) - 1) * this.scales;
        ys = (j - parseInt(this.image_size / 2) - 1) * this.scales;
        for (var i1 = 0; i1 < this.grid_nodes_number; i1++)
            for (var j1 = 0; j1 < this.grid_nodes_number; j1++)
            {
                x = (i1 - parseInt(this.grid_nodes_number / 2) - 1) * pixel; 
                y = (j1 - parseInt(this.grid_nodes_number / 2) - 1) * pixel;
                rk = Math.sqrt(k * k * ((x - xs) * (x - xs) + (y - ys) * (y - ys) + this.d_L * this.d_L));
                r2 = ((x - xs) * (x - xs) + (y - ys) * (y - ys) + this.d_L * this.d_L);
                ImPart += te1* this.inter_sc[i1][j1] * Math.sin(rk) / r2;
				RePart += te1* this.inter_sc[i1][j1] * Math.cos(rk) / r2;
            }
        this.sc[i][j] = (ImPart*ImPart + RePart*RePart)/this.la/this.la;
		if (this.sc[i][j] < this.minIntense)
            {
                this.minIntense = this.sc[i][j];
                this.mini = i; 
                this.minj = j;
            }
            if (this.sc[i][j] > this.maxIntense)
            {
                this.maxIntense = this.sc[i][j];
                this.maxi = i;
                this.maxj = j;
            }
       // if (scr[scrc]!=0.0) console.log('yes');
		}*/
		
		
    for (var scrc = 0; scrc < scenter; scrc++)
    {
        sum = 0.0;
		RePart = 0.0;
		ImPart = 0.0;
        xs = scrc * this.scales;
        ys = xs;
        for (var i1 = 0; i1 < this.grid_nodes_number; i1++)
            for (var j1 = 0; j1 < this.grid_nodes_number; j1++)
            {
                x = (i1 - pcenter) * pixel; 
                y = (j1 - pcenter) * pixel;
                rk = Math.sqrt(k * k * ((x - xs) * (x - xs) + (y - ys) * (y - ys) + this.d_L * this.d_L));
                r2 = ((x - xs) * (x - xs) + (y - ys) * (y - ys) + this.d_L * this.d_L);
                ImPart += te1* this.inter_sc[i1][j1] * Math.sin(rk) / r2;
				RePart += te1* this.inter_sc[i1][j1] * Math.cos(rk) / r2;
            }
        scr[scrc] = (ImPart*ImPart + RePart*RePart)/this.la/this.la;
       // if (scr[scrc]!=0.0) console.log('yes');
    }
    koe = Math.sqrt(2);
    for (var i = 0; i < this.image_size; i++)
        for (var j = 0; j < this.image_size; j++)
        {
            rk = Math.sqrt((i + 0.5 - scenter) * (i + 0.5 - scenter) + (j + 0.5 - scenter) * (j + 0.5 - scenter)) / koe;
            inte = Math.floor(rk);
            re = rk - inte;
            if (inte < scenter - 1) this.sc[i][j] = scr[inte] * (1.0 - re) + scr[inte + 1] * re;
            else if (inte < scenter) this.sc[i][j] = scr[inte];
            else this.sc[i][j] = 0.0;
            if (this.sc[i][j] < this.minIntense)
            {
                this.minIntense = this.sc[i][j];
                this.mini = i; 
                this.minj = j;
            }
            if (this.sc[i][j] > this.maxIntense)
            {
                this.maxIntense = this.sc[i][j];
                this.maxi = i;
                this.maxj = j;
            }
        }
    for (var i = 0; i < this.image_size; i++)
        for (var j = 0; j < this.image_size; j++)
        {
			if ((((i-scenter)*(i-scenter)+(j-scenter)*(j-scenter))*this.scales*this.scales<d_radius*d_radius*1.25*1.25)||(this.fresnel_zones<3))
			{
				vis = (this.sc[i][j] - this.minIntense) / this.maxIntense;
				this.DiffractionImage[i][j] = {
					r: parseInt(r * 2*vis), 
					g: parseInt(g *  2*vis), 
					b: parseInt(b *  2*vis),
					a: 1,
					val: vis
				};
			}
			else 
			{
				this.DiffractionImage[i][j] = {r:0,g:0,b:0,a:0,val:0};
				vis=0;
			}
			var rad = parseInt(d_radius/this.scales); //Pixels for radius
			if (((i-scenter) * (i-scenter) + (j-scenter) * (j-scenter) < rad*rad)&&
			((i-scenter)*(i-scenter)+(j-scenter)*(j-scenter)>(rad-1)*(rad-1)))
				this.DiffractionImage[i][j] = {r:255,g:255,b:255,a:1,val:vis};
		}
		//this.DiffractionImage=this.InterferentionImage;
	var stmin = d_la*1000000/Math.sqrt(d_radius*d_radius*4*1000000000000 - d_la*1000000000000*d_la)*d_L;
    this.isInitDiffraction = true;
    return this.DiffractionImage;
};

function MakeSplines(x, y , N , xc) 
{
	var h = [],l = [],delta = [],lambda = [],d = [],c = [],b = [],a = [];
	var k;
	for(k=1; k<=N; k++){
       h[k] = x[k] - x[k-1];
	   l[k] = (y[k] - y[k-1])/h[k];
	}
	delta[1] = - h[2]/(2*(h[1]+h[2]));
	lambda[1] = 1.5*(l[2] - l[1])/(h[1]+h[2]);
	for(k=3; k<=N; k++){
		delta[k-1] = - h[k]/(2*h[k-1] + 2*h[k] + h[k-1]*delta[k-2]);
		lambda[k-1] = (3*l[k] - 3*l[k-1] - h[k-1]*lambda[k-2]) /
			(2*h[k-1] + 2*h[k] + h[k-1]*delta[k-2]);
	}
	c[0] = 0;
	c[N] = 0;
	for(k=N; k>=2; k--){
		c[k-1] = delta[k-1]*c[k] + lambda[k-1];
	}
	for(k=1; k<=N; k++){
		a[k] = y[k];
		d[k] = (c[k] - c[k-1])/(3*h[k]);
		b[k] = l[k] + (2*c[k]*h[k] + h[k]*c[k-1])/3;
	}
	var splined = [];
	var maxg = 0.0;
	for (var i=0;i<N-1;i++) {
		splined[6*i] = {
			x:xc[6*i]*Math.sqrt(2),
			y:y[i]
		}
		if (splined[6*i].y>maxg)
			maxg = splined[6*i].y;
		for(var j=1;j<=5;j++)
		{
			splined[6*i+j] = {
				x:xc[6*i+j]*Math.sqrt(2.0),
				y:Math.abs(a[i+1] + (xc[6*i+j]  - x[i+1])*b[i+1] + (xc[6*i+j]  - x[i+1])*(xc[6*i+j]  - x[i+1])*c[i+1] + (xc[6*i+j]  - x[i+1])*(xc[6*i+j]  - x[i+1])*(xc[6*i+j]  - x[i+1])*d[i+1])
			}
			if (splined[6*i+j].y>maxg)
			maxg = splined[6*i+j].y;
		}//splined[6*i] = a[i+1] + xc[i]*b[i+1] + xc[i]*xc[i]*c[i+1] + xc[i]*xc[i]*xc[i]*d[i+1]
	}
	for (i=0;i<N-1;i++)
		for (j=0;j<6;j++)
			splined[6*i+j].y/=maxg;
	
	return splined;
}

InterAndDiffr.prototype.CalcDiffr = function (d_L,d_la, max_size)
{
    this.d_L = d_L;
    this.d_E = this.E * this.tau;
    this.d_la = d_la;
    this.sc = [];
    for(var i = 0; i < this.image_size; i++) this.sc[i] = [];
    this.DiffractionImage = [];
    for(var i = 0; i < this.image_size; i++) this.DiffractionImage[i] = [];
    var scr = [];
	
	this.fresnel_zones = (this.lin_size*this.lin_size/4.0/d_la/d_L);
	console.log(this.fresnel_zones+ ' '+this.lin_size);
    
	if (this.fresnel_zones>50)
		this.grid_nodes_number = 1001;
	else if (this.fresnel_zones>25) 
		this.grid_nodes_number = 501;
	else if (this.fresnel_zones>12) 
		this.grid_nodes_number = 251;
	else 
		this.grid_nodes_number = 351;
	this.CalcInterGrid();
	
	var pixel = this.pixel_scale;
    this.scales = 1.20*max_size / this.image_size;
	
	
    var k = 2.0 * Math.PI / this.d_la;
    var E0 = this.E * this.tau;
    var sum, xs, ys, x, y, koe, re, inte, vis;
    this.size_m = this.scales * this.image_size;
    this.minIntense = E0 * E0;
    this.maxIntense = 0.0;
	var RePart;
	var ImPart , te, te1;
	te1 = pixel*pixel*E0*this.d_L;
	var center = parseInt(this.grid_nodes_number / 2)-1;
	
	var pcenter = this.grid_nodes_number/2;//parseInt(this.grid_nodes_number/2)+1;
	var scenter = this.image_size/2;
	var scenter1 = this.image_size/2+6;
	var te1 = pixel*pixel*E0*this.d_L;
	var scr1 = [];
	var ctct = 0;
    for (var scrc = 0; scrc < scenter1; scrc+=6)
    {
		ctct++;
        sum = 0.0;
		RePart = 0.0;
		ImPart = 0.0;
        xs = scrc * this.scales;
        ys = xs;
        for (var i1 = 0; i1 < this.grid_nodes_number; i1++)
            for (var j1 = 0; j1 < this.grid_nodes_number; j1++)
            {
                x = (i1 - pcenter) * pixel; 
                y = (j1 - pcenter) * pixel;
                rk = Math.sqrt(k * k * ((x - xs) * (x - xs) + (y - ys) * (y - ys) + this.d_L * this.d_L));
                r2 = ((x - xs) * (x - xs) + (y - ys) * (y - ys) + this.d_L * this.d_L);
                ImPart += te1* this.inter_sc[i1][j1] * Math.sin(rk) / r2;
				RePart += te1* this.inter_sc[i1][j1] * Math.cos(rk) / r2;
            }
        scr1[scrc] = (ImPart*ImPart + RePart*RePart)/this.la/this.la;
       // if (scr[scrc]!=0.0) console.log('yes');
    }
	var scr2 = [];
	var x1 = [],x2 = [];
	for (i=0;i<ctct;i++) {
		x1[i] = i*this.scales*6;
		scr2[i] = scr1[i*6];
	}
	for (i=0;i<scenter;i++)
	{
		x2[i] = i*this.scales;
	}	
	//console.log(scr2);
	this.GraphicInterpolated = MakeSplines(x1,scr2,ctct-1,x2);
	console.log(this.GraphicInterpolated);
	for (i=0;i<this.GraphicInterpolated.length;i++)
		if (this.GraphicInterpolated[i] == undefined)
			console.log("OOOOPS" + i);
	//console.log(ctct);
	//	console.log(scr);

	//scr[scrc] = scr[scrc+1] = scr[scrc+2] = scr1[scrc];
		/*scr[scrc] = scr1[scrc];
		scr[scrc+1] = 0.66667*scr1[scrc] + 0.33333*scr1[scrc+3];
		scr[scrc+2] = 0.66667*scr1[scrc+3] + 0.33333*scr1[scrc];*/
	
	
	for (var scrc = 0; scrc < scenter; scrc+=6)
    {
		
		scr[scrc] = scr1[scrc];
		scr[scrc+1] = 0.833333333*scr1[scrc] + 0.16666667* scr1[scrc+6];
		scr[scrc+2] = 0.666666667*scr1[scrc] + 0.333333333*scr1[scrc+6];
		scr[scrc+3] = 0.5*scr1[scrc] + 0.5*scr1[scrc+6];
		scr[scrc+4] = 0.333333333*scr1[scrc] + 0.666666667*scr1[scrc+6];
		scr[scrc+5] = 0.16666667*scr1[scrc] + 0.833333333*scr1[scrc+6];

	}
    koe = Math.sqrt(2);
    for (var i = 0; i < this.image_size; i++)
        for (var j = 0; j < this.image_size; j++)
        {
            rk = Math.sqrt((i-scenter)*(i-scenter)+(j-scenter)*(j-scenter))/koe;//Math.sqrt((i + 0.5 - scenter) * (i + 0.5 - scenter) + (j + 0.5 - scenter) * (j + 0.5 - scenter)) / koe;
            inte = Math.floor(rk);
            re = rk - inte;
            if (inte < scenter - 1) this.sc[i][j] = scr[inte] * (1.0 - re) + scr[inte + 1] * re;
            else if (inte < scenter) this.sc[i][j] = scr[inte];
            else this.sc[i][j] = 0.0;
			//this.sc[i][j] = scr[inte];
            if (this.sc[i][j] < this.minIntense)
            {
                this.minIntense = this.sc[i][j];
                this.mini = i; 
                this.minj = j;
            }
            if (this.sc[i][j] > this.maxIntense)
            {
                this.maxIntense = this.sc[i][j];
                this.maxi = i;
                this.maxj = j;
            }
        }
	console.log(this.maxi+ ' ' + this.maxj);
    for (var i = 0; i < this.image_size; i++)
        for (var j = 0; j < this.image_size; j++)
        {
            vis = (this.sc[i][j]) / this.maxIntense;
            this.DiffractionImage[i][j] = vis;
        }
    this.isInitDiffraction = true;
    return this.DiffractionImage;
};
InterAndDiffr.prototype.CountScale = function(NumberOfRings, la, l, size)
{
    var n1 = 1.0 + NumberOfRings;
    return res = 2.0 * Math.sqrt(n1 * n1 * la * la + 2 * n1 * la * l) / size;
};
InterAndDiffr.prototype.CountInterMatveev = function(la, l, e0, x, y, psize, tau, e2)
{
    var e1 = e0 * tau;
    var x1 = x * psize;
    var y1 = y * psize;
    var r = Math.sqrt(l * l + x1 * x1 + y1 * y1);
    return e1 * e1 + e2 * e2 / (r * r) + 2 * e2 * e1 * Math.cos(-1 * (x1 * x1 + y1 * y1) * Math.PI / (la * l)) / r;
};
InterAndDiffr.prototype.CountColor = function(la, R, G, B)
{
    var la1 = parseInt(la * 1000000000.0);
    R = G = B = 0;
	if ((la1>560)&&(la1<=760))   R = 255;
	if ((la1>495)&&(la1<=555))   R = 0;
	if ((la1>570)&&(la1<=580))   R=Math.round(127.5+127.5*(la1-570)/10);
	if ((la1>555)&&(la1<=570))   R=Math.round(127.5*(la1-555)/15);
	if ((la1>480)&&(la1<=495))   R=Math.round(127.5-127.5*(la1-480)/15);
	if ((la1>=380)&&(la1<=480))   R=Math.round(255-127.5*(la1-380)/100);

	if ((la1>=380)&&(la1<=525))   B=255;
	if ((la1>555)&&(la1<=760))   B=0;
	if ((la1>540)&&(la1<=555))   B=Math.round(127.5-127.5*(la1-540)/15);
	if ((la1>525)&&(la1<=540))   B=Math.round(255-127.5*(la1-525)/15);
	
	if ((la1>525)&&(la1<=580))   G=255;
	if ((la1>=380)&&(la1<=495))  G=0;
	if ((la1>610)&&(la1<=760))   G=Math.round(63.5-63.5*(la1-610)/150);
	if ((la1>600)&&(la1<=610))  G=Math.round(127.5-63.5*(la1-600)/10);
	if ((la1>590)&&(la1<=600))   G=Math.round(190.5-63.5*(la1-590)/10);
	if ((la1>580)&&(la1<=590))   G=Math.round(255-63.5*(la1-580)/10);
	if ((la1>495)&&(la1<=510))   G=Math.round(127.5*(la1-495)/15);
	if ((la1>510)&&(la1<=525))   G=Math.round(127.5+127.5*(la1-510)/15);

	
	return { r: R, g: G, b: B };
}
/*InterAndDiffr.prototype.CountColor = function(la, R, G, B)
{
    var la1 = parseInt(la * 1000000000.0);
    R = G = B = 0;
    if ((la1 <= 480) && (la1 >= 380)) { R = 200; B = 200; }
    else if ((la1 > 480) && (la1 <= 510)) B = 200;
    else if ((la1 > 510) && (la1 <= 540)) { G = 200; B = 200; }
    else if ((la1 > 540) && (la1 <= 570)) G = 200;
    else if ((la1 > 570) && (la1 <= 590)) { R = 200; G = 100; }
    else if ((la1 > 590) && (la1 <= 610)) { R = 200; G = 100; }
    else if ((la1 > 610) && (la1 <= 760)) R = 200;
    else if ((la1 > 760) || (la1 < 380)) return null;
    return { r: R, g: G, b: B };
};*/